/*
 Copyright 2011 George Rhoten

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */
package com.centralnexus.thermal;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Window;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.RenderedImage;
import java.awt.image.WritableRaster;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Formatter;
import java.util.Hashtable;
import java.util.Scanner;
import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;

/**
 *
 * @author George Rhoten
 * @author Max Ritter
 */
public class Output extends javax.swing.JPanel implements MouseListener, MouseMotionListener {

	private static final long serialVersionUID = 6966468502962301998L;

	private static final short SETTINGS_XPADDING = 32;
	private static final short DEFAULT_THERMAL_WIDTH = 640;
    private static final short DEFAULT_THERMAL_HEIGHT = 480;

	private MousePositionTracker mt;
    private double[][] temperatures;
    private int[][] redvalues;
    private int[][] greenvalues;
    private int[][] bluevalues;
    private boolean[][] showtemp;
    private double max;
    private double average;
    private double min;
    private double tmin;
    private double tmax;
    private double step;
    private int values;
    private int rows;
    private int lines;
    private int broadness;
    private int roomtemp;
    private String name;
    private String dirName;

    /** Creates new form OutputNew */
    public Output(Window parent) {
    	super();
        initComponents();
    }

    public void run(String dirName, int rows, int lines) {
        mt = new MousePositionTracker();
        addMouseMotionListener(mt);
        addMouseMotionListener(this);
        addMouseListener(this);
        this.dirName = dirName;
        this.rows = rows;
        this.lines = lines;
        values = rows * lines;
        this.broadness = (int) Math.round(DEFAULT_THERMAL_WIDTH / rows);
        temperatures = new double[rows * 2][lines * 2];
        redvalues = new int[rows * 2][lines * 2];
        greenvalues = new int[rows * 2][lines * 2];
        bluevalues = new int[rows * 2][lines * 2];
        showtemp = new boolean[rows * 2][lines * 2];
        TemperatureData();
        LimitValues();
        IntToRGB();
        thermalimage.setIcon(new ImageIcon(convertRenderedImage(CreateThermalImage())));
        BufferedImage wcamimage;
        try {
            wcamimage = ImageIO.read(new File(dirName + "/Webcam.jpg"));
        } catch (Exception ex) {
            System.out.println("Exception : " + ex.getMessage());
            wcamimage = new BufferedImage(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT, BufferedImage.TYPE_INT_RGB);
        }
        webcamimage.setIcon(new ImageIcon(convertRenderedImage(wcamimage)));
        colourscale.setIcon(new ImageIcon(convertRenderedImage(createColorScale())));
        resolution.setText(rows + " x " + lines);
        mouseovertemp.setText("N/A");
        formatTemperatureStatistics();
        maxslider.setPaintLabels(true);
        maxslider.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                calculateSlider();
            }
        });

        minslider.setPaintLabels(true);
        minslider.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                calculateSlider();
            }
        });
//        JOptionPane.showMessageDialog(null, "Left click on a place in the image to place a temperature marker at that location.",
//                "Help", JOptionPane.INFORMATION_MESSAGE);
//        repaint();
    }

    private void initComponents() {
        GridBagConstraints mainConstraints = new GridBagConstraints();
        GridBagConstraints settingsConstraints = new GridBagConstraints();

        //setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        //setTitle("Image Results");
        setBackground(java.awt.SystemColor.window);
        //setResizable(false);
        setLayout(new GridBagLayout());

        thermalimage.setBackground(Color.WHITE);
        thermalimage.setBorder(javax.swing.BorderFactory.createLineBorder(Color.BLACK));
        thermalimage.setPreferredSize(new java.awt.Dimension(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT));

        mainConstraints.insets = new Insets(4, 4, 4, 4);
        mainConstraints.fill = GridBagConstraints.BOTH;
        add(thermalimage, mainConstraints);

        webcamimage.setBackground(Color.WHITE);
        webcamimage.setBorder(javax.swing.BorderFactory.createLineBorder(Color.BLACK));
        webcamimage.setPreferredSize(new java.awt.Dimension(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT));

        mainConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        add(webcamimage, mainConstraints);


        settingsPanel.setBackground(Color.WHITE);
        settingsPanel.setBorder(javax.swing.BorderFactory.createLineBorder(Color.BLACK));
        settingsPanel.setLayout(new GridBagLayout());

        Insets defaultInsets = settingsConstraints.insets;
        settingsConstraints.ipadx = SETTINGS_XPADDING;

        minsliderlabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        minsliderlabel.setText("Minimum Temp:");
        settingsConstraints.insets = new Insets(8, 0, 0, 0);
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(minsliderlabel, settingsConstraints);

        minslider.setBackground(Color.WHITE);
        settingsConstraints.anchor = GridBagConstraints.CENTER;
        settingsPanel.add(minslider, settingsConstraints);

//        settingsConstraints.gridheight = 2;
        settingsConstraints.gridwidth = 2;
        settingsPanel.add(colourscale, settingsConstraints);
//        settingsConstraints.gridheight = 1; // Reset to the default
        settingsConstraints.gridwidth = 1; // Reset to the default

        averagetemplabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        averagetemplabel.setText("Average Temp:");
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(averagetemplabel, settingsConstraints);

        averagetemp.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsConstraints.ipadx = 0;
        settingsPanel.add(averagetemp, settingsConstraints);
        settingsConstraints.ipadx = SETTINGS_XPADDING;

        mouseovertemplabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        mouseovertemplabel.setText("Mouseover Temp:");
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(mouseovertemplabel, settingsConstraints);

        mouseovertemp.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsConstraints.ipadx = 0;
        settingsConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        settingsPanel.add(mouseovertemp, settingsConstraints);
        settingsConstraints.ipadx = SETTINGS_XPADDING;
        settingsConstraints.gridwidth = 1; // Reset to the default

        maxsliderlabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        maxsliderlabel.setText("Maximum Temp:");
        settingsConstraints.insets = defaultInsets;
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(maxsliderlabel, settingsConstraints);

        maxslider.setBackground(Color.WHITE);
        settingsPanel.add(maxslider, settingsConstraints);

        defaultscale.setBackground(Color.WHITE);
        scaleTypeGroup.add(defaultscale);
        defaultscale.setFont(new Font(Font.DIALOG, Font.BOLD, 14)); // NOI18N
        defaultscale.setSelected(true);
        defaultscale.setText("Blue Green Red");
        defaultscale.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	calculateSlider();
            }
        });
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(defaultscale, settingsConstraints);

        alternativescale.setBackground(Color.WHITE);
        scaleTypeGroup.add(alternativescale);
        alternativescale.setFont(new Font(Font.DIALOG, Font.BOLD, 14)); // NOI18N
        alternativescale.setText("Red Yellow");
        alternativescale.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	calculateSlider();
            }
        });
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(alternativescale, settingsConstraints);

        maxtemplabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        maxtemplabel.setText("Maximum Temp:");
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(maxtemplabel, settingsConstraints);

        maxtemp.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(maxtemp, settingsConstraints);

        ambienttemplabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        ambienttemplabel.setText("Ambient Temp:");
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(ambienttemplabel, settingsConstraints);

        ambienttemp.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(ambienttemp, settingsConstraints);
        settingsConstraints.gridwidth = 1; // Reset to the default

        save.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        save.setText("Save");
        save.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	SaveIMG();
            }
        });
        settingsConstraints.insets = new Insets(0, 0, 8, 0);
        settingsConstraints.anchor = GridBagConstraints.CENTER;
        settingsPanel.add(save, settingsConstraints);

        complete.setBackground(Color.WHITE);
        complete.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        complete.setText("Combined Image");
        settingsPanel.add(complete, settingsConstraints);

        celsiusButton.setBackground(Color.WHITE);
        temperatureTypeGroup.add(celsiusButton);
        celsiusButton.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        celsiusButton.setSelected(true);
        celsiusButton.setText("Celsius");
        celsiusButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	formatTemperatureStatistics();
            }
        });
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(celsiusButton, settingsConstraints);

        fahrenheitButton.setBackground(Color.WHITE);
        temperatureTypeGroup.add(fahrenheitButton);
        fahrenheitButton.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        fahrenheitButton.setText("Fahrenheit");
        fahrenheitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	formatTemperatureStatistics();
            }
        });
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(fahrenheitButton, settingsConstraints);

        mintemplabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        mintemplabel.setText("Minimum Temp:");
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(mintemplabel, settingsConstraints);

        mintemp.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(mintemp, settingsConstraints);

        resolutionlabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        resolutionlabel.setText("Resolution:");
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(resolutionlabel, settingsConstraints);

        resolution.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(resolution, settingsConstraints);

        mainConstraints.gridwidth = 2;
        add(settingsPanel, mainConstraints);
    }

    private JPanel settingsPanel = new JPanel();
    private JLabel ambienttemp = new JLabel();
    private JLabel ambienttemplabel = new JLabel();
    private JLabel averagetemp = new JLabel();
    private JLabel averagetemplabel = new JLabel();
    private JRadioButton defaultscale = new JRadioButton();
    private JRadioButton alternativescale = new JRadioButton();
    private ButtonGroup scaleTypeGroup = new ButtonGroup();
    private JRadioButton celsiusButton = new JRadioButton();
    private JRadioButton fahrenheitButton = new JRadioButton();
    private ButtonGroup temperatureTypeGroup = new ButtonGroup();
    private JLabel colourscale = new JLabel();
    private JCheckBox complete = new JCheckBox();
    private JSlider maxslider = new JSlider();
    private JLabel maxsliderlabel = new JLabel();
    private JLabel maxtemp = new JLabel();
    private JLabel maxtemplabel = new JLabel();
    private JSlider minslider = new JSlider();
    private JLabel minsliderlabel = new JLabel();
    private JLabel mintemp = new JLabel();
    private JLabel mintemplabel = new JLabel();
    private JLabel mouseovertemp = new JLabel();
    private JLabel mouseovertemplabel = new JLabel();
    private JLabel resolution = new JLabel();
    private JLabel resolutionlabel = new JLabel();
    private JButton save = new JButton();
    private JLabel thermalimage = new JLabel();
    private JLabel webcamimage = new JLabel();

    public void CalculateMouseTemp() {
        int xpos = (int) mt.getXPosition() - thermalimage.getX() - 5;
        int ypos = (int) mt.getYPosition() - thermalimage.getY() - 20;
        int xtempnummer = (int) Math.round(((xpos * 10.0) / (broadness * 0.5)) / 10.0) - 1;
        int ytempnummer = (int) (Math.round(((ypos * 10.0) / (broadness * 0.5)) / 10.0)) - 1;
        if ((xtempnummer < ((rows * 2) - 1)) && (ytempnummer < ((lines * 2) - 1)) && (xtempnummer >= 0) && (ytempnummer >= 0)) {
            mouseovertemp.setText(formatTemperature(temperatures[xtempnummer][ytempnummer], true));
        } else {
            mouseovertemp.setText("N/A");
        }
        repaint();
    }

    public void SetshowTemp() {
        int xpos = (int) mt.getXPosition() - thermalimage.getX();
        int ypos = (int) mt.getYPosition() - thermalimage.getY() - 35;
        int xtempnummer = (int) Math.round(((xpos * 10.0) / (broadness * 0.5)) / 10.0) - 2;
        int ytempnummer = (int) (Math.round(((ypos * 10.0) / (broadness * 0.5)) / 10.0)) + 1;
        if ((xtempnummer < ((rows * 2) - 1)) && (ytempnummer < ((lines * 2) - 1)) && (xtempnummer > 0) && (ytempnummer > 0)) {
            if (!showtemp[xtempnummer][ytempnummer] && !showtemp[xtempnummer - 1][ytempnummer]
                    && !showtemp[xtempnummer - 1][ytempnummer - 1] && !showtemp[xtempnummer - 1][ytempnummer + 1]
                    && !showtemp[xtempnummer][ytempnummer - 1] && !showtemp[xtempnummer][ytempnummer + 1]
                    && !showtemp[xtempnummer + 1][ytempnummer] && !showtemp[xtempnummer + 1][ytempnummer - 1]
                    && !showtemp[xtempnummer + 1][ytempnummer + 1])
            {
                showtemp[xtempnummer][ytempnummer] = true;
            }
            else {
                showtemp[xtempnummer + 1][ytempnummer + 1] = false;
                showtemp[xtempnummer + 1][ytempnummer] = false;
                showtemp[xtempnummer + 1][ytempnummer - 1] = false;
                showtemp[xtempnummer][ytempnummer + 1] = false;
                showtemp[xtempnummer][ytempnummer] = false;
                showtemp[xtempnummer][ytempnummer - 1] = false;
                showtemp[xtempnummer - 1][ytempnummer + 1] = false;
                showtemp[xtempnummer - 1][ytempnummer] = false;
                showtemp[xtempnummer - 1][ytempnummer - 1] = false;
            }
        }
        calculateSlider();
    }

    public void LimitValues() {
        max = 0.0;
        min = temperatures[0][0];
        average = 0.0;
        for (int x = 0; x < ((rows * 2) - 1); x = x + 2) {
            for (int y = 0; y < ((lines * 2) - 1); y = y + 2) {
                if (temperatures[x][y] > max) {
                    max = temperatures[x][y];
                }
                if (temperatures[x][y] < min) {
                    min = temperatures[x][y];
                }
                average += temperatures[x][y];
            }
        }
        average = average / (rows * lines);
        tmax = max;
        tmin = min;
    }

    public void IntToRGB() {
        int redVal, greenVal, blueVal;
        step = (((max - min) * 100.0) / 10) / 100.0;

        if (defaultscale.isSelected()) {
            for (int x = 0; x < ((rows * 2) - 1); x++) {
                for (int y = 0; y < ((lines * 2) - 1); y++) {
                    double temperature = temperatures[x][y];
                    if (temperature < min) {
                        redVal = 28;
                        greenVal = 1;
                        blueVal = 108;
                    }
                    else if (temperature <= (min + step)) {
                        redVal = calculateRGB(28, 31, min, (min + step), temperature);
                        greenVal = calculateRGB(1, 17, min, (min + step), temperature);
                        blueVal = calculateRGB(108, 218, min, (min + step), temperature);
                    }
                    else if (temperature <= (min + 2 * step)) {
                        redVal = calculateRGB(31, 50, (min + step), (min + 2 * step), temperature);
                        greenVal = calculateRGB(17, 111, (min + step), (min + 2 * step), temperature);
                        blueVal = calculateRGB(218, 238, (min + step), (min + 2 * step), temperature);
                    }
                    else if (temperature <= (min + 3 * step)) {
                        redVal = calculateRGB(50, 63, (min + 2 * step), (min + 3 * step), temperature);
                        greenVal = calculateRGB(111, 196, (min + 2 * step), (min + 3 * step), temperature);
                        blueVal = calculateRGB(238, 229, (min + 2 * step), (min + 3 * step), temperature);
                    }
                    else if (temperature <= (min + 4 * step)) {
                        redVal = calculateRGB(63, 64, (min + 3 * step), (min + 4 * step), temperature);
                        greenVal = calculateRGB(196, 222, (min + 3 * step), (min + 4 * step), temperature);
                        blueVal = calculateRGB(229, 135, (min + 3 * step), (min + 4 * step), temperature);
                    }
                    else if (temperature <= (min + 5 * step)) {
                        redVal = calculateRGB(64, 192, (min + 4 * step), (min + 5 * step), temperature);
                        greenVal = calculateRGB(222, 240, (min + 4 * step), (min + 5 * step), temperature);
                        blueVal = calculateRGB(135, 14, (min + 4 * step), (min + 5 * step), temperature);
                    }
                    else if (temperature <= (min + 6 * step)) {
                        redVal = calculateRGB(192, 223, (min + 5 * step), (min + 6 * step), temperature);
                        greenVal = calculateRGB(240, 172, (min + 5 * step), (min + 6 * step), temperature);
                        blueVal = calculateRGB(14, 18, (min + 5 * step), (min + 6 * step), temperature);
                    }
                    else if (temperature <= (min + 7 * step)) {
                        redVal = calculateRGB(223, 209, (min + 6 * step), (min + 7 * step), temperature);
                        greenVal = calculateRGB(172, 111, (min + 6 * step), (min + 7 * step), temperature);
                        blueVal = calculateRGB(18, 14, (min + 6 * step), (min + 7 * step), temperature);
                    }
                    else if (temperature <= (min + 8 * step)) {
                        redVal = calculateRGB(209, 210, (min + 7 * step), (min + 8 * step), temperature);
                        greenVal = calculateRGB(111, 50, (min + 7 * step), (min + 8 * step), temperature);
                        blueVal = calculateRGB(14, 28, (min + 7 * step), (min + 8 * step), temperature);
                    }
                    else if (temperature <= (min + 9 * step)) {
                        redVal = calculateRGB(210, 194, (min + 8 * step), (min + 9 * step), temperature);
                        greenVal = calculateRGB(50, 26, (min + 8 * step), (min + 9 * step), temperature);
                        blueVal = calculateRGB(28, 0, (min + 8 * step), (min + 9 * step), temperature);
                    }
                    else if (temperature <= max) {
                        redVal = calculateRGB(194, 132, (min + 9 * step), max, temperature);
                        greenVal = calculateRGB(26, 26, (min + 9 * step), max, temperature);
                        blueVal = calculateRGB(0, 0, (min + 9 * step), max, temperature);
                    }
                    else { // if (temperature > max)
                        redVal = 132;
                        greenVal = 26;
                        blueVal = 0;
                    }
                    redvalues[x][y] = redVal;
                    greenvalues[x][y] = greenVal;
                    bluevalues[x][y] = blueVal;
                }
            }
        }
        else if (alternativescale.isSelected()) {
            for (int x = 0; x < ((rows * 2) - 1); x++) {
                for (int y = 0; y < ((lines * 2) - 1); y++) {
                    double temperature = temperatures[x][y];
                    if (temperature < min) {
                        redVal = 0;
                        greenVal = 0;
                        blueVal = 5;
                    }
                    else if (temperature <= (min + step)) {
                        redVal = calculateRGB(0, 7, min, (min + step), temperature);
                        greenVal = calculateRGB(0, 1, min, (min + step), temperature);
                        blueVal = calculateRGB(5, 97, min, (min + step), temperature);
                    }
                    else if (temperature <= (min + 2 * step)) {
                        redVal = calculateRGB(7, 51, (min + step), (min + 2 * step), temperature);
                        greenVal = calculateRGB(1, 1, (min + step), (min + 2 * step), temperature);
                        blueVal = calculateRGB(97, 194, (min + step), (min + 2 * step), temperature);
                    }
                    else if (temperature <= (min + 3 * step)) {
                        redVal = calculateRGB(51, 110, (min + 2 * step), (min + 3 * step), temperature);
                        greenVal = calculateRGB(1, 2, (min + 2 * step), (min + 3 * step), temperature);
                        blueVal = calculateRGB(194, 212, (min + 2 * step), (min + 3 * step), temperature);
                    }
                    else if (temperature <= (min + 4 * step)) {
                        redVal = calculateRGB(110, 158, (min + 3 * step), (min + 4 * step), temperature);
                        greenVal = calculateRGB(2, 6, (min + 3 * step), (min + 4 * step), temperature);
                        blueVal = calculateRGB(212, 150, (min + 3 * step), (min + 4 * step), temperature);
                    }
                    else if (temperature <= (min + 5 * step)) {
                        redVal = calculateRGB(158, 197, (min + 4 * step), (min + 5 * step), temperature);
                        greenVal = calculateRGB(6, 30, (min + 4 * step), (min + 5 * step), temperature);
                        blueVal = calculateRGB(150, 58, (min + 4 * step), (min + 5 * step), temperature);
                    }
                    else if (temperature <= (min + 6 * step)) {
                        redVal = calculateRGB(197, 218, (min + 5 * step), (min + 6 * step), temperature);
                        greenVal = calculateRGB(30, 66, (min + 5 * step), (min + 6 * step), temperature);
                        blueVal = calculateRGB(58, 0, (min + 5 * step), (min + 6 * step), temperature);
                    }
                    else if (temperature <= (min + 7 * step)) {
                        redVal = calculateRGB(218, 237, (min + 6 * step), (min + 7 * step), temperature);
                        greenVal = calculateRGB(66, 137, (min + 6 * step), (min + 7 * step), temperature);
                        blueVal = calculateRGB(0, 0, (min + 6 * step), (min + 7 * step), temperature);
                    }
                    else if (temperature <= (min + 8 * step)) {
                        redVal = calculateRGB(237, 246, (min + 7 * step), (min + 8 * step), temperature);
                        greenVal = calculateRGB(137, 199, (min + 7 * step), (min + 8 * step), temperature);
                        blueVal = calculateRGB(0, 23, (min + 7 * step), (min + 8 * step), temperature);
                    }
                    else if (temperature <= (min + 9 * step)) {
                        redVal = calculateRGB(246, 251, (min + 8 * step), (min + 9 * step), temperature);
                        greenVal = calculateRGB(199, 248, (min + 8 * step), (min + 9 * step), temperature);
                        blueVal = calculateRGB(23, 117, (min + 8 * step), (min + 9 * step), temperature);
                    }
                    else if (temperature <= max) {
                        redVal = calculateRGB(251, 252, (min + 9 * step), max, temperature);
                        greenVal = calculateRGB(248, 254, (min + 9 * step), max, temperature);
                        blueVal = calculateRGB(117, 243, (min + 9 * step), max, temperature);
                    }
                    else { // if (temperature > max)
                        redVal = 252;
                        greenVal = 254;
                        blueVal = 243;
                    }
                    redvalues[x][y] = redVal;
                    greenvalues[x][y] = greenVal;
                    bluevalues[x][y] = blueVal;
                }
            }
        }
    }

    private static int calculateRGB(int rgb1, int rgb2, double t1, double t2, double t) {
        return (int)(rgb1 + (((((t - t1) * 100.0) / (t2 - t1)) / 100.0) * (rgb2 - rgb1)));
    }

    public void SaveIMG() {
        BufferedImage img1 = convertRenderedImage(CreateThermalImage());
        img1 = img1.getSubimage(0, 0, img1.getWidth(), img1.getHeight());
        img1 = img1.getSubimage(0, 0, img1.getWidth(), img1.getHeight());

        BufferedImage img2 = convertRenderedImage(createColorScale());

        BufferedImage combined = null;
        Graphics g = null;
        if (complete.isSelected()) {
            BufferedImage wcamimage = new BufferedImage(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT, BufferedImage.TYPE_INT_RGB);
            try {
                wcamimage = ImageIO.read(new File(dirName + "/Webcam.jpg"));
            }
            catch (Exception ex) {
                System.out.println("Exception : " + ex.getMessage());
            }
            combined = new BufferedImage(img1.getWidth() + wcamimage.getWidth() + 30, img1.getHeight() + 80,
                    BufferedImage.TYPE_INT_RGB);
            g = combined.getGraphics();
            g.setColor(Color.WHITE);
            g.fillRect(0, 0, img1.getWidth() + wcamimage.getWidth() + 30, img1.getHeight() + 80);
            g.drawImage(wcamimage, img1.getWidth() + 20, 10, null);
        }
        else {
            combined = new BufferedImage(img1.getWidth() + 20, img1.getHeight() + 80, BufferedImage.TYPE_INT_RGB);
            g = combined.getGraphics();
            g.setColor(Color.WHITE);
            g.fillRect(0, 0, img1.getWidth() + 20, img1.getHeight() + 80);
        }

        int xadd = (int) Math.round(((img1.getWidth() - 550) * 0.5) + 0.5);

        Font font = new Font("Times New Roman", Font.BOLD, 15);
        g.setFont(font);
        SimpleDateFormat sdf = new SimpleDateFormat();
        g.setColor(Color.darkGray);
        g.drawString(name, 218 + xadd, img1.getHeight() + 30);
        g.drawImage(img1, 10, 10, null);
        g.drawImage(img2, xadd - 25, img1.getHeight() + 20, null);
        g.setFont(new Font("Times New Roman", Font.PLAIN, 14));
        g.drawString(sdf.format(new Date()), 480 + xadd, img1.getHeight() + 30);
        g.drawString("Room temp.: " + formatTemperature(roomtemp, true), 218 + xadd, img1.getHeight() + 55);
        g.drawString("Min. temp.: " + formatTemperature(tmin, true), 350 + xadd, img1.getHeight() + 55);
        g.drawString("Max. temp.: " + formatTemperature(tmax, true), 480 + xadd, img1.getHeight() + 55);
        g.dispose();

        try {
            JFileChooser fc = new JFileChooser();
            fc.setCurrentDirectory(new File(dirName));
            fc.removeChoosableFileFilter(fc.getAcceptAllFileFilter());
            fc.setFileFilter(new JPGfilter());
            fc.setSelectedFile(new File("ThermalImage.jpg"));
            int rVal = fc.showDialog(null, "Save");
            if (rVal == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();
                ImageIO.write(combined, "jpg", file);
            }
        } catch (IOException e) {
        	e.printStackTrace(System.err);
        }
    }

    public RenderedImage CreateThermalImage() {
        double xWidthFactor = broadness * 0.5;
        double yWidthFactor = broadness * 0.5;
        BufferedImage bufferedImage = new BufferedImage(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2d = bufferedImage.createGraphics();

        g2d.setColor(Color.WHITE);
        g2d.fillRect(0, 0, DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT);
        g2d.addRenderingHints(new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON));
        for (int x = 0; x < ((rows * 2) - 1); x++) {
            for (int y = 0; y < ((lines * 2) - 1); y++) {
                g2d.setColor(new Color(redvalues[x][y], greenvalues[x][y], bluevalues[x][y], 255));
                g2d.fillRect((int) (x * xWidthFactor) + 2, (int) (y * yWidthFactor) + 2, (int) Math.round(xWidthFactor), (int) Math.round(yWidthFactor));
            }
        }
        // Draw text on top of color background.
        g2d.setFont(new Font("Arial", Font.BOLD, 15));
        for (int x = 0; x < ((rows * 2) - 1); x++) {
            for (int y = 0; y < ((lines * 2) - 1); y++) {
                if (showtemp[x][y]) {
                    int ix = (int) ((x + 1) * xWidthFactor);
                    int iy = (int) ((y + 1) * yWidthFactor);
                    g2d.setColor(Color.black);
                    g2d.fillOval(ix, iy, (int) Math.round(xWidthFactor), (int) Math.round(yWidthFactor));
                    g2d.drawString(formatTemperature(temperatures[x][y], true), ix - 20, iy + 10 + broadness);
                }
            }
        }
        g2d.dispose();

        return bufferedImage;
    }

    public RenderedImage createColorScale() {
        BufferedImage bufferedImage = new BufferedImage(230, 32, BufferedImage.TYPE_INT_RGB);
        try {
            if (defaultscale.isSelected()) {
                bufferedImage = ImageIO.read(this.getClass().getResource("Colorscale1.png"));
            } else if (alternativescale.isSelected()) {
                bufferedImage = ImageIO.read(this.getClass().getResource("Colorscale2.png"));
            }
        } catch (Exception ex) {
            System.out.println("Exception : " + ex.getMessage());
        }
        Graphics2D g2d = bufferedImage.createGraphics();
        g2d.addRenderingHints(new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON));
        g2d.setFont(new Font("Arial", Font.BOLD, 12));
        g2d.setColor(Color.black);
        g2d.drawString(formatTemperature(min, false), 3, 30);
        g2d.drawString(formatTemperature(min + step, false), 26, 30);
        g2d.drawString(formatTemperature(min + 2 * step, false), 49, 30);
        g2d.drawString(formatTemperature(min + 3 * step, false), 72, 30);
        g2d.drawString(formatTemperature(min + 4 * step, false), 95, 30);
        g2d.drawString(formatTemperature(min + 5 * step, false), 118, 30);
        g2d.drawString(formatTemperature(min + 6 * step, false), 141, 30);
        g2d.drawString(formatTemperature(min + 7 * step, false), 164, 30);
        g2d.drawString(formatTemperature(min + 8 * step, false), 187, 30);
        g2d.drawString(formatTemperature(max, false), 212, 30);
        g2d.dispose();

        return bufferedImage;
    }

    public void TemperatureData() {
        try {
            Scanner scanner = new Scanner(new File(dirName + "\\tempvalues.txt"));
            int i = 0;
            for (int x = ((rows * 2) - 2); x >= 0; x = x - 2) {
                for (int y = ((lines * 2) - 2); y >= 0; y = y - 2) {
                    if ((scanner.hasNextDouble()) && (i <= values)) {
                        temperatures[x][y] = scanner.nextDouble();
                        i++;
                    }
                }
            }
            roomtemp = scanner.nextInt();
            name = scanner.next();
        } catch (IOException e) {
        	e.printStackTrace(System.err);
        }

        for (int x = 0; x < ((rows * 2) - 1); x = x + 2) {
            for (int y = 1; y < ((lines * 2) - 1); y = y + 2) {
                temperatures[x][y] = ((temperatures[x][y + 1] + temperatures[x][y - 1]) * 0.5);
            }
        }
        for (int x = 1; x < ((rows * 2) - 1); x = x + 2) {
            for (int y = 0; y < ((lines * 2) - 1); y = y + 2) {
                temperatures[x][y] = ((temperatures[x + 1][y] + temperatures[x - 1][y]) * 0.5);
            }
        }
        for (int x = 1; x < ((rows * 2) - 1); x = x + 2) {
            for (int y = 1; y < ((lines * 2) - 1); y = y + 2) {
                temperatures[x][y] = ((temperatures[x + 1][y] + temperatures[x - 1][y] + temperatures[x][y + 1] + temperatures[x][y - 1]) * 0.25);
            }
        }
    }

    public BufferedImage convertRenderedImage(RenderedImage img) {
        if (img instanceof BufferedImage) {
            return (BufferedImage) img;
        }
        ColorModel cm = img.getColorModel();
        int width = img.getWidth();
        int height = img.getHeight();
        WritableRaster raster = cm.createCompatibleWritableRaster(width, height);
        boolean isAlphaPremultiplied = cm.isAlphaPremultiplied();
        Hashtable<String, Object> properties = new Hashtable<String, Object>();
        String[] keys = img.getPropertyNames();
        if (keys != null) {
            for (int i = 0; i < keys.length; i++) {
                properties.put(keys[i], img.getProperty(keys[i]));
            }
        }
        BufferedImage result = new BufferedImage(cm, raster, isAlphaPremultiplied, properties);
        img.copyData(raster);
        return result;
    }

	private void formatTemperatureStatistics() {
		ambienttemp.setText(formatTemperature(roomtemp, true));
        averagetemp.setText(formatTemperature(average, true));
        mintemp.setText(formatTemperature(tmin, true));
        maxtemp.setText(formatTemperature(tmax, true));

        int middletemp;
        if ((roomtemp > tmin) && (roomtemp < tmax)) {
            middletemp = roomtemp;
        } else {
            middletemp = (int) Math.round(tmin + ((tmax - tmin) * 0.5));
        }
        int maxVal = (int) Math.round(convertToCurrentTemperatureType(tmax + 25));
        int minVal = (int) Math.round(convertToCurrentTemperatureType(middletemp + 1));
        int val = (int) Math.round(convertToCurrentTemperatureType(tmax));
        int bigspace = (int) ((maxVal - minVal) * 0.20);
        int smallspace = (int) (0.5 * bigspace);
        maxslider.setMinimum(val - smallspace);
        maxslider.setMaximum(val + bigspace * 4);
        maxslider.setValue(val);
        maxslider.setMinorTickSpacing(smallspace);
        maxslider.setMajorTickSpacing(bigspace);

        maxVal = (int) Math.round(convertToCurrentTemperatureType(middletemp - 1));
        minVal = (int) Math.round(convertToCurrentTemperatureType(tmin - 25));
        val = (int) Math.round(convertToCurrentTemperatureType(tmin));
        minslider.setMinimum(minVal);
        minslider.setMaximum(maxVal);
        minslider.setValue(val);
        bigspace = (int) ((maxVal - minVal) * 0.20);
        smallspace = (int) (0.5 * bigspace);
        minslider.setMinimum(val - bigspace * 4);
        minslider.setMaximum(val + smallspace);
        minslider.setValue(val);
        minslider.setMinorTickSpacing(smallspace);
        minslider.setMajorTickSpacing(bigspace);
        calculateSlider();
	}

    public void calculateSlider() {
        max = convertFromCurrentTemperatureType(maxslider.getValue());
        min = convertFromCurrentTemperatureType(minslider.getValue());
        IntToRGB();
        colourscale.setIcon(new ImageIcon(convertRenderedImage(createColorScale())));
        thermalimage.setIcon(new ImageIcon(convertRenderedImage(CreateThermalImage())));
    }

    private String formatTemperature(double val, boolean showLongFormat) {
        StringBuilder sb = new StringBuilder();
        Formatter formatter = new Formatter(sb);
        String format;
        if (showLongFormat) {
            format = "%1$.2f\u00B0%2$s";
        }
        else {
            format = "%1$.0f\u00B0";
        }

        if (celsiusButton.isSelected()) {
            formatter.format(format, val, (showLongFormat ? "C" : ""));
        }
        else {
            formatter.format(format, convertToFahrenheit(val), (showLongFormat ? "F" : ""));
        }
        return sb.toString();
    }

    private static double convertToFahrenheit(double val) {
        return val * 9 / 5 + 32;
    }
    
    private static double convertFromFahrenheit(double val) {
        return (val - 32) * 5 / 9;
    }

    /**
     * 
     * @param val Celsius
     * @return Celsius or Fahrenheit
     */
    private double convertToCurrentTemperatureType(double val) {
        if (celsiusButton.isSelected()) {
            return val;
        }
        else {
            return convertToFahrenheit(val);
        }
    }

    /**
     * 
     * @param val Celsius or Fahrenheit
     * @return Celsius
     */
    private double convertFromCurrentTemperatureType(double val) {
        if (celsiusButton.isSelected()) {
            return val;
        }
        else {
            return convertFromFahrenheit(val);
        }
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        if (mt.getMouseButton(e) == MouseEvent.BUTTON1) {
            SetshowTemp();
        }
    }

    @Override
    public void mouseExited(MouseEvent e) {
    }

    @Override
    public void mousePressed(MouseEvent e) {
    }

    @Override
    public void mouseReleased(MouseEvent e) {
    }

    @Override
    public void mouseEntered(MouseEvent e) {
    }

	@Override
	public void mouseDragged(MouseEvent arg0) {
	}

	@Override
	public void mouseMoved(MouseEvent e) {
        CalculateMouseTemp();
	}

	private class MousePositionTracker extends MouseMotionAdapter {

        private Point pos;

        @Override
        public synchronized void mouseDragged(MouseEvent e) {
            pos = e.getPoint();
        }

        @Override
        public synchronized void mouseMoved(MouseEvent e) {
            pos = e.getPoint();
        }

        public synchronized double getXPosition() {
            if (pos == null) {
                return 0;
            }
            return pos.getX();
        }

        public synchronized double getYPosition() {
            if (pos == null) {
                return 0;
            }
            return pos.getY();
        }

        public synchronized int getMouseButton(MouseEvent e) {
            return e.getButton();
        }
    }

    public class JPGfilter extends FileFilter {

        /* (non-Javadoc)
         * @see filechooser.FileFilter#accept(java.io.File)
         */
        @Override
        public boolean accept(File f) {
            return (f.getName().toLowerCase().endsWith(".jpg")
                    || f.getName().toLowerCase().endsWith(".jpeg"));
        }

        @Override
        public String getDescription() {
            return "jpg/jpeg Image";
        }
    }
}
