/*
 Copyright 2011 George Rhoten

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */
package com.centralnexus.thermal;

import com.lti.civil.CaptureDeviceInfo;
import com.lti.civil.CaptureException;
import com.lti.civil.CaptureObserver;
import com.lti.civil.CaptureStream;
import com.lti.civil.CaptureSystem;
import com.lti.civil.CaptureSystemFactory;
import com.lti.civil.DefaultCaptureSystemFactorySingleton;
import com.lti.civil.awt.AWTImageConverter;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Vector;
import javax.imageio.ImageIO;
import javax.swing.*;

/**
 *
 * @author George Rhoten
 * @author Max Ritter
 */
public class WebcamPanel extends JComponent implements Runnable {

	private static final long serialVersionUID = 3132607519545989880L;
	private static CaptureSystem system;
    private CaptureStream captureStream;
    private static CaptureSystemFactory factory;
    private CaptureDeviceInfo captureInfo = null;
    private volatile boolean disposing = false;
    // Default image in case the camera is missing.
    private Image image = new BufferedImage(640, 480, BufferedImage.TYPE_INT_RGB);

    public WebcamPanel() {
        initComponents();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        GroupLayout layout = new GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGap(0, 400,
                Short.MAX_VALUE));
        layout.setVerticalGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGap(0, 300,
                Short.MAX_VALUE));
    }
    
    public void run(String deviceID) throws CaptureException {
        initCapture(deviceID);
        if (captureStream != null) {
            setSize(captureStream.getVideoFormat().getWidth(), captureStream.getVideoFormat().getHeight());
        }

        setVisible(true);
        startCapture();
    }

    public void saveImage(String dirName) throws CaptureException {
        saveJPG(dirName);
        disposeCapture();
    }

    /**
     * Initialize the capture given the index of the given camera.
     * @param devID Use null for first instance or provide the system dependent camera identifier.
     * @throws CaptureException
     */
	public void initCapture(String devID) throws CaptureException {
        List<?> list = getCaptureList();
        if (list.size() > 0) {
            if (devID == null || devID.length() == 0) {
                captureInfo = ((CaptureDeviceInfo) list.get(0));
            }
            else {
                for (int i = 0; i < list.size(); ++i) {
                    CaptureDeviceInfo currInfo = ((CaptureDeviceInfo) list.get(i));
                    if (currInfo.getDeviceID().equals(devID)) {
                        captureInfo = currInfo;
                        break;
                    }
                }
            }
        }
        if (captureInfo != null) {
            System.out.println("Device ID: " + captureInfo.getDeviceID());
            System.out.println("Description: " + captureInfo.getDescription());
            captureStream = system.openCaptureDeviceStream(captureInfo.getDeviceID());
            captureStream.setObserver(new MyCaptureObserver());
        }
        else {
            System.err.println("No video cameras present.");
        }
    }
	
	public static synchronized List<CaptureDeviceInfo> getCaptureList() {
        Vector<CaptureDeviceInfo> retList = new Vector<CaptureDeviceInfo>();
        try {
            if (factory == null) {
                factory = DefaultCaptureSystemFactorySingleton.instance();
            }
            if (system == null) {
                system = factory.createCaptureSystem();
            }
	        system.init();
	        List<?> list = system.getCaptureDeviceInfoList();
	        for (int i = 0; i < list.size(); ++i) {
	            CaptureDeviceInfo info = ((CaptureDeviceInfo) list.get(i));
	            retList.add(info);
	        }
		} catch (CaptureException e) {
            e.printStackTrace(System.err);
		}
        return retList;
	}

	public String getDeviceID() {
	    return captureInfo != null ? captureInfo.getDeviceID() : "";
	}
	
    public void startCapture() throws CaptureException {
        if (captureStream != null) {
            captureStream.start();
        } else {
            JOptionPane.showMessageDialog(null, "The visual spectrum camera is not connected.\n"
                    + "Please connect the camera and restart the program.", "Warning", JOptionPane.WARNING_MESSAGE);
        }
    }

    public void disposeCapture() throws CaptureException {
        disposing = true;

        if (captureStream != null) {
            //System.out.println("disposeCapture: stopping capture stream...");
            captureStream.stop();
            //System.out.println("disposeCapture: stopped capture stream.");
            captureStream.dispose();
            captureStream = null;
        }

        if (system != null) {
            system.dispose();
            system = null;
        }
        disposing = false;
        //System.out.println("disposeCapture done.");
    }

    class MyCaptureObserver implements CaptureObserver {

        @Override
        public void onError(CaptureStream sender, CaptureException e) {
            e.printStackTrace();
        }

        @Override
        public void onNewImage(CaptureStream sender, com.lti.civil.Image image) {
            if (disposing) {
                return;
            }
            try {
                setImage(AWTImageConverter.toBufferedImage(image));
            } catch (Throwable t) {
                t.printStackTrace();
            }
        }
    }

    public synchronized void setImage(Image image) {
        this.image = image;
        SwingUtilities.invokeLater(this);
    }

    @Override
    public void run() {
        setPreferredSize(new Dimension(image.getWidth(null), image.getHeight(null)));
        repaint();
    }

    public void saveJPG(String dirName) {
        try {
            ImageIO.write((BufferedImage) image, "jpg", new File(dirName + "/Webcam.jpg"));
        } catch (IOException ex) {
            ex.printStackTrace(System.err);
        }
    }

    @Override
    public synchronized void paint(Graphics g) {
        if (image != null) {
            g.drawImage(image, 0, 0, this);
            g.setColor(Color.yellow);
            g.fillOval(310, 230, 20, 20);
            g.fillOval(0, 470, 20, 20);
        }
    }
}
